{% extends "lockdown/base.html" %}

{% block title %}Coming soon...{% endblock %}

{% block content %}
<div id="lockdown">
  <h2>Coming soon...</h2>

  <p>This is not yet available to the public.</p>

  {% if form %}
  <form action="" method="post">{% csrf_token %}
    {{ form.as_p }}
    <p><input type="submit" value="Preview" /></p>
  </form>
  {% endif %}

</div>
{% endblock %}
